@@ -2,7 +2,8 @@
 #'
 #' Compute the consensus ranking using either cumulative probability (CP) or maximum a posteriori (MAP) consensus
 #' \insertCite{vitelli2018}{BayesMallows}. For mixture models, the
-#' consensus is given for each mixture.
+#' consensus is given for each mixture. Consensus of augmented ranks can also be computed
+#' for each assessor, by setting \code{parameter = "Rtilde"}.
 #'
 #' @param model_fit An object returned from \code{\link{compute_mallows}}.
 #'
@@ -13,6 +14,14 @@
 #' to discard as burn-in. Defaults to \code{model_fit$burnin}, and must be
 #' provided if \code{model_fit$burnin} does not exist. See \code{\link{assess_convergence}}.
 #'
+#' @param parameter Character string defining the parameter for which to compute the
+#' consensus. Defaults to \code{"rho"}. Available options are \code{"rho"} and \code{"Rtilde"},
+#' with the latter giving consensus rankings for augmented ranks.
+#'
+#' @param assessors When \code{parameter = "rho"}, this integer vector is used to
+#' define the assessors for which to compute the augmented ranking. Defaults to
+#' \code{1L}, which yields augmented rankings for assessor 1.
+#'
 #' @references \insertAllCited{}
 #'
 #'
@@ -20,41 +29,79 @@
 #'
 #' @example /inst/examples/compute_consensus_example.R
 #'
-compute_consensus <- function(model_fit, type = "CP", burnin = model_fit$burnin){
-
-  if(type == "CP"){
-    .compute_cp_consensus(model_fit, burnin = burnin)
-  } else if(type == "MAP"){
-    .compute_map_consensus(model_fit, burnin = burnin)
-  }
-
-
-}
-
-.compute_cp_consensus <- function(model_fit, burnin = model_fit$burnin){
-
-  stopifnot(class(model_fit) == "BayesMallows")
+compute_consensus <- function(model_fit, type = "CP", burnin = model_fit$burnin,
+                              parameter = "rho", assessors = 1L){
 
   if(is.null(burnin)){
     stop("Please specify the burnin.")
   }
-
   stopifnot(burnin < model_fit$nmc)
 
-  # Filter out the pre-burnin iterations
-  df <- dplyr::filter(model_fit$rho, .data$iteration > burnin)
+  stopifnot(class(model_fit) == "BayesMallows")
 
-  # Find the problem dimensions
-  n_rows <- nrow(dplyr::distinct(df, .data$item, .data$cluster))
+  if(parameter == "Rtilde" && !inherits(model_fit$augmented_data, "data.frame")){
+    stop(paste("For augmented ranks, please refit",
+               "model with option 'save_aug = TRUE'."))
+  }
 
-  # Check that there are rows.
-  stopifnot(n_rows > 0)
+  if(parameter == "rho"){
+    # Filter out the pre-burnin iterations
+    df <- dplyr::filter(model_fit$rho, .data$iteration > burnin)
 
-  # Check that the number of rows are consistent with the information in
-  # the model object
-  stopifnot(model_fit$n_clusters * model_fit$n_items == n_rows)
+    # Find the problem dimensions
+    n_rows <- nrow(dplyr::distinct(df, .data$item, .data$cluster))
 
-  # Convert items and clustr to character, since factor levels are not needed in this case
+    # Check that there are rows.
+    stopifnot(n_rows > 0)
+
+    # Check that the number of rows are consistent with the information in
+    # the model object
+    stopifnot(model_fit$n_clusters * model_fit$n_items == n_rows)
+
+    df <- if(type == "CP"){
+      .compute_cp_consensus(df)
+    } else if(type == "MAP"){
+      .compute_map_consensus(df)
+    }
+
+
+  } else if(parameter == "Rtilde"){
+    # Filter out the pre-burnin iterations and get the right assessors
+    df <- dplyr::filter(model_fit$augmented_data, .data$iteration > burnin, .data$assessor %in% assessors)
+
+    # Find the problem dimensions
+    n_rows <- nrow(dplyr::distinct(df, .data$assessor, .data$item))
+
+    # Check that there are rows.
+    stopifnot(n_rows > 0)
+
+    # Check that the number of rows are consistent with the information in
+    # the model object
+    stopifnot(length(assessors) * model_fit$n_items == n_rows)
+
+    # Treat assessors as clusters
+    df <- dplyr::rename(df, cluster = "assessor")
+
+    df <- if(type == "CP"){
+      .compute_cp_consensus(df)
+    } else if(type == "MAP"){
+      .compute_map_consensus(df)
+    }
+
+    if("cluster" %in% names(df)){
+      df <- dplyr::rename(df, assessor = "cluster")
+    }
+
+  }
+
+  return(df)
+
+
+}
+
+.compute_cp_consensus <- function(df){
+
+  # Convert items and cluster to character, since factor levels are not needed in this case
   df <- dplyr::mutate_at(df, dplyr::vars(.data$item, .data$cluster),
                          as.character)
 
@@ -80,13 +127,12 @@ compute_consensus <- function(model_fit, type = "CP", burnin = model_fit$burnin)
   df <- dplyr::ungroup(df)
 
   # If there is only one cluster, we drop the cluster column
-  if(model_fit$n_clusters == 1){
+  if(length(unique(df$cluster)) == 1){
     df <- dplyr::select(df, -.data$cluster)
   }
 
   return(df)
-
-  }
+}
 
 
 # Internal function for finding CP consensus.
@@ -109,7 +155,7 @@ find_cpc <- function(group_df){
 
     # Keep the max only. This filtering must be done after the first filter,
     # since we take the maximum among the filtered values
-    tmp_df <- dplyr::filter(tmp_df, .data$cumprob == max(.data$cumprob))
+    tmp_df <- dplyr::filter(tmp_df, .data$cumprob == max(dplyr::coalesce(.data$cumprob, 0)))
 
     # Add the ranking
     tmp_df <- dplyr::mutate(tmp_df, ranking = i)
@@ -122,19 +168,13 @@ find_cpc <- function(group_df){
   return(result)
 }
 
-.compute_map_consensus <- function(model_fit, burnin = model_fit$burnin){
-
-  if(is.null(burnin)){
-    stop("Please specify the burnin.")
-  }
-
-  df <- dplyr::filter(model_fit$rho, .data$iteration > burnin)
+.compute_map_consensus <- function(df){
 
   # Store the total number of iterations after burnin
   n_samples <- length(unique(df$iteration))
 
   # Spread to get items along columns
-  df <- tidyr::spread(df, key = .data$item, value = .data$value)
+  df <- tidyr::pivot_wider(df, names_from = "item", values_from = "value")
 
   # Group by everything except iteration, and count the unique combinations
   df <- dplyr::group_by_at(df, .vars = dplyr::vars(-.data$iteration))
@@ -151,16 +191,17 @@ find_cpc <- function(group_df){
   df <- dplyr::select(df, -.data$n_max, -.data$n)
 
   # Now collect one set of ranks per cluster
-  df <- tidyr::gather(df, key = "item", value = "map_ranking",
-                      -.data$cluster, -.data$probability)
+  df <- tidyr::pivot_longer(df, cols = -c("cluster", "probability"), names_to = "item", values_to = "map_ranking")
 
   # Sort according to cluster and ranking
   df <- dplyr::arrange(df, .data$cluster, .data$map_ranking)
 
-  if(model_fit$n_clusters == 1){
+  if(length(unique(df$cluster)) == 1){
     df <- dplyr::select(df, -.data$cluster)
   }
 
+
+
   return(df)
 
 }
